#include <JGL/FontCache.h>
#include <iostream>

using namespace JGL;

char CachedGlyph::getCharacter() {
    return character;
}

const GLuint* CachedGlyph::getTexture() {
    return &texture;
}

CachedGlyph::CachedGlyph(GLuint texture_id, char c, float x2offset, float y2offset, float w, float h, float advanceX, float advanceY) {
    texture = texture_id;
    character = c;
    this->x2offset = x2offset;
    this->y2offset = y2offset;
    this->w = w;
    this->h = h;
    this->advanceX = advanceX;
    this->advanceY = advanceY;
}

//TODO
//Because most things shown would be english characters. We can cut down on the iteration time significantly
//by putting each english character at the beginning of the list in order of how often they usually occur in text.
void JGL::CachedFont::appendGlyph(JGL::CachedGlyph* glyph) {
    glyphs.push_back(glyph);
}

unsigned int JGL::CachedFont::getFontSize() {
    return font_size;
}

unsigned int JGL::CachedFont::getFontIndex() {
    return font_index;
}

CachedGlyph* JGL::CachedFont::getGlyph(char c) {
    for (const auto& g : glyphs)
        if (c == g->getCharacter())
            return g;
    return nullptr;
}

CachedFont::CachedFont(unsigned int font_size, unsigned int font_index) {
    this->font_size = font_size;
    this->font_index = font_index;
}

void CachedFont::eraseGlyph(CachedGlyph* glyph) {
    if (glyph == nullptr)
        return;

    for (int i = 0; i < glyphs.size(); i++)
        if (glyphs[i] == glyph)
            glDeleteTextures(1, glyphs[i]->getTexture()),
            delete glyphs[i],
            glyphs.erase(glyphs.begin() + i);
}

void CachedFont::eraseGlyph(char c) {
    for (int i = 0; i < glyphs.size(); i++)
        if (glyphs[i]->getCharacter() == c)
            glDeleteTextures(1, glyphs[i]->getTexture()),
            delete glyphs[i],
            glyphs.erase(glyphs.begin() + i);
}

void CachedFont::eraseGlyph(GLuint texture_id) {
    for (int i = 0; i < glyphs.size(); i++)
        if (glyphs[i]->getTexture() == &texture_id)
            glDeleteTextures(1, glyphs[i]->getTexture()),
            delete glyphs[i],
            glyphs.erase(glyphs.begin() + i);
}

std::vector<CachedGlyph*> CachedFont::getGlyphs() {
    return glyphs;
}

void FontCache::appendFont(CachedFont* font) {
    cachedFonts.push_back(font);
}

void FontCache::newFont(unsigned int font_size, unsigned int font_index) {
    auto* font = new CachedFont(font_size, font_index);
    cachedFonts.push_back(font);
}

void FontCache::eraseFont(CachedFont* font) {
    for (int i = 0; i < cachedFonts.size(); i++) {
        if (cachedFonts[i] == font) {
            for (auto& g: cachedFonts[i]->getGlyphs())
                cachedFonts[i]->eraseGlyph(g);

            delete cachedFonts[i];
            cachedFonts.erase(cachedFonts.begin() + i);
        }
    }
}

void FontCache::purgeCache() {
    //Remove every font from the cache.
    for (const auto& font : cachedFonts)
        eraseFont(font);
}

std::vector<CachedFont*> FontCache::getFonts() {
    return cachedFonts;
}

CachedFont* FontCache::getFont(unsigned int font_size, unsigned int font_index) {
    if (cachedFonts.empty())
        return nullptr;

    for (auto* f : cachedFonts)
        if (f->getFontIndex() == font_index && f->getFontSize() == font_size)
            return f;
    return nullptr;
}
